@@ -28,6 +28,7 @@ import itertools
 import logging
 import sys
 import os
+import gc
 
 from ryu import cfg
 from ryu import utils
@@ -146,7 +147,7 @@ class RyuApp(object):
         """
         Return iterator over the (key, contxt class) of application context
         """
-        return cls._CONTEXTS.iteritems()
+        return iter(cls._CONTEXTS.items())
 
     def __init__(self, *_args, **_kwargs):
         super(RyuApp, self).__init__()
@@ -242,7 +243,7 @@ class RyuApp(object):
 
     def get_observers(self, ev, state):
         observers = []
-        for k, v in self.observers.get(ev.__class__, {}).iteritems():
+        for k, v in self.observers.get(ev.__class__, {}).items():
             if not state or not v or state in v:
                 observers.append(k)
 
@@ -284,12 +285,12 @@ class RyuApp(object):
         if name in SERVICE_BRICKS:
             if isinstance(ev, EventRequestBase):
                 ev.src = self.name
-            LOG.debug("EVENT %s->%s %s" %
-                      (self.name, name, ev.__class__.__name__))
+            LOG.debug("EVENT %s->%s %s",
+                      self.name, name, ev.__class__.__name__)
             SERVICE_BRICKS[name]._send_event(ev, state)
         else:
-            LOG.debug("EVENT LOST %s->%s %s" %
-                      (self.name, name, ev.__class__.__name__))
+            LOG.debug("EVENT LOST %s->%s %s",
+                      self.name, name, ev.__class__.__name__)
 
     def send_event_to_observers(self, ev, state=None):
         """
@@ -344,6 +345,10 @@ class AppManager(object):
             hub.joinall(services)
         finally:
             app_mgr.close()
+            for t in services:
+                t.kill()
+            hub.joinall(services)
+            gc.collect()
 
     @staticmethod
     def get_instance():
@@ -375,8 +380,7 @@ class AppManager(object):
         while len(app_lists) > 0:
             app_cls_name = app_lists.pop(0)
 
-            context_modules = map(lambda x: x.__module__,
-                                  self.contexts_cls.values())
+            context_modules = [x.__module__ for x in self.contexts_cls.values()]
             if app_cls_name in context_modules:
                 continue
 
@@ -423,7 +427,7 @@ class AppManager(object):
             for _k, m in inspect.getmembers(i, inspect.ismethod):
                 if not hasattr(m, 'callers'):
                     continue
-                for ev_cls, c in m.callers.iteritems():
+                for ev_cls, c in m.callers.items():
                     if not c.ev_source:
                         continue
 
@@ -433,18 +437,18 @@ class AppManager(object):
                                                 c.dispatchers)
 
                     # allow RyuApp and Event class are in different module
-                    for brick in SERVICE_BRICKS.itervalues():
+                    for brick in SERVICE_BRICKS.values():
                         if ev_cls in brick._EVENTS:
                             brick.register_observer(ev_cls, i.name,
                                                     c.dispatchers)
 
     @staticmethod
     def _report_brick(name, app):
-        LOG.debug("BRICK %s" % name)
+        LOG.debug("BRICK %s", name)
         for ev_cls, list_ in app.observers.items():
-            LOG.debug("  PROVIDES %s TO %s" % (ev_cls.__name__, list_))
+            LOG.debug("  PROVIDES %s TO %s", ev_cls.__name__, list_)
         for ev_cls in app.event_handlers.keys():
-            LOG.debug("  CONSUMES %s" % (ev_cls.__name__,))
+            LOG.debug("  CONSUMES %s", ev_cls.__name__)
 
     @staticmethod
     def report_bricks():
@@ -511,5 +515,7 @@ class AppManager(object):
                 self._close(app)
             close_dict.clear()
 
-        close_all(self.applications)
+        for app_name in list(self.applications.keys()):
+            self.uninstantiate(app_name)
+        assert not self.applications
         close_all(self.contexts)
