@@ -31,6 +31,7 @@ import traceback
 import random
 import ssl
 from socket import IPPROTO_TCP, TCP_NODELAY
+import warnings
 
 import ryu.base.app_manager
 
@@ -120,11 +121,30 @@ class Datapath(ofproto_protocol.ProtocolDesc):
 
         self.xid = random.randint(0, self.ofproto.MAX_XID)
         self.id = None  # datapath_id is unknown yet
-        self.ports = None
+        self._ports = None
         self.flow_format = ofproto_v1_0.NXFF_OPENFLOW10
         self.ofp_brick = ryu.base.app_manager.lookup_service_brick('ofp_event')
         self.set_state(handler.HANDSHAKE_DISPATCHER)
 
+    def _get_ports(self):
+        if (self.ofproto_parser is not None and
+                self.ofproto_parser.ofproto.OFP_VERSION >= 0x04):
+            message = (
+                'Datapath#ports is kept for compatibility with the previous '
+                'openflow versions (< 1.3). '
+                'This not be updated by EventOFPPortStatus message. '
+                'If you want to be updated, you can use '
+                '\'ryu.controller.dpset\' or \'ryu.topology.switches\'.'
+            )
+            warnings.warn(message, stacklevel=2)
+        return self._ports
+
+    def _set_ports(self, ports):
+        self._ports = ports
+
+    # To show warning when Datapath#ports is read
+    ports = property(_get_ports, _set_ports)
+
     def close(self):
         self.set_state(handler.DEAD_DISPATCHER)
 
@@ -145,6 +165,7 @@ class Datapath(ofproto_protocol.ProtocolDesc):
             ret = self.socket.recv(required_len)
             if len(ret) == 0:
                 self.is_active = False
+                self.socket.close()
                 break
             buf += ret
             while len(buf) >= required_len:
