@@ -20,7 +20,7 @@ import itertools
 
 from ryu import exception
 from ryu.lib import mac
-from . import ofproto_parser
+from ryu.lib.pack_utils import msg_pack_into
 from . import ofproto_v1_0
 from . import inet
 
@@ -90,6 +90,7 @@ class Flow(object):
         self.nw_frag = 0
         self.regs = [0] * FLOW_N_REGS
         self.ipv6_label = 0
+        self.pkt_mark = 0
 
 
 class FlowWildcards(object):
@@ -111,6 +112,7 @@ class FlowWildcards(object):
         self.regs_bits = 0
         self.regs_mask = [0] * FLOW_N_REGS
         self.wildcards = ofproto_v1_0.OFPFW_ALL
+        self.pkt_mark_mask = 0
 
 
 class ClsRule(object):
@@ -257,20 +259,16 @@ class ClsRule(object):
         self.wc.wildcards &= ~FWW_IPV6_LABEL
         self.flow.ipv6_label = label
 
-    def set_ipv6_label(self, label):
-        self.wc.wildcards &= ~FWW_IPV6_LABEL
-        self.flow.ipv6_label = label
-
     def set_ipv6_src_masked(self, src, mask):
         self.wc.ipv6_src_mask = mask
-        self.flow.ipv6_src = [x & y for (x, y) in itertools.izip(src, mask)]
+        self.flow.ipv6_src = [x & y for (x, y) in zip(src, mask)]
 
     def set_ipv6_src(self, src):
         self.flow.ipv6_src = src
 
     def set_ipv6_dst_masked(self, dst, mask):
         self.wc.ipv6_dst_mask = mask
-        self.flow.ipv6_dst = [x & y for (x, y) in itertools.izip(dst, mask)]
+        self.flow.ipv6_dst = [x & y for (x, y) in zip(dst, mask)]
 
     def set_ipv6_dst(self, dst):
         self.flow.ipv6_dst = dst
@@ -278,7 +276,7 @@ class ClsRule(object):
     def set_nd_target_masked(self, target, mask):
         self.wc.nd_target_mask = mask
         self.flow.nd_target = [x & y for (x, y) in
-                               itertools.izip(target, mask)]
+                               zip(target, mask)]
 
     def set_nd_target(self, target):
         self.flow.nd_target = target
@@ -291,6 +289,10 @@ class ClsRule(object):
         self.flow.regs[reg_idx] = value
         self.wc.regs_bits |= (1 << reg_idx)
 
+    def set_pkt_mark_masked(self, pkt_mark, mask):
+        self.flow.pkt_mark = pkt_mark
+        self.wc.pkt_mark_mask = mask
+
     def flow_format(self):
         # Tunnel ID is only supported by NXM
         if self.wc.tun_id_mask != 0:
@@ -308,6 +310,9 @@ class ClsRule(object):
         if not self.wc.wildcards & FWW_NW_ECN:
             return ofproto_v1_0.NXFF_NXM
 
+        if self.wc.regs_bits > 0:
+            return ofproto_v1_0.NXFF_NXM
+
         return ofproto_v1_0.NXFF_OPENFLOW10
 
     def match_tuple(self):
@@ -437,7 +442,7 @@ class MFField(object):
         return cls(header, value, mask)
 
     def _put(self, buf, offset, value):
-        ofproto_parser.msg_pack_into(self.pack_str, buf, offset, value)
+        msg_pack_into(self.pack_str, buf, offset, value)
         return self.n_bytes
 
     def putw(self, buf, offset, value, mask):
@@ -456,8 +461,7 @@ class MFField(object):
             return self.putw(buf, offset, value, mask)
 
     def _putv6(self, buf, offset, value):
-        ofproto_parser.msg_pack_into(self.pack_str, buf, offset,
-                                     *value)
+        msg_pack_into(self.pack_str, buf, offset, *value)
         return self.n_bytes
 
     def putv6(self, buf, offset, value, mask):
@@ -912,6 +916,17 @@ class MFRegister(MFField):
                     return self._put(buf, offset, rule.flow.regs[i])
 
 
+@_register_make
+@_set_nxm_headers([ofproto_v1_0.NXM_NX_PKT_MARK, ofproto_v1_0.NXM_NX_PKT_MARK_W])
+class MFPktMark(MFField):
+    @classmethod
+    def make(cls, header):
+        return cls(header, MF_PACK_STRING_BE32)
+
+    def put(self, buf, offset, rule):
+        return self.putm(buf, offset, rule.flow.pkt_mark, rule.wc.pkt_mark_mask)
+
+
 def serialize_nxm_match(rule, buf, offset):
     old_offset = offset
 
@@ -1069,6 +1084,13 @@ def serialize_nxm_match(rule, buf, offset):
             header = ofproto_v1_0.NXM_NX_IP_FRAG_W
         offset += nxm_put(buf, offset, header, rule)
 
+    if rule.flow.pkt_mark != 0:
+        if rule.wc.pkt_mark_mask == UINT32_MAX:
+            header = ofproto_v1_0.NXM_NX_PKT_MARK
+        else:
+            header = ofproto_v1_0.NXM_NX_PKT_MARK_W
+        offset += nxm_put(buf, offset, header, rule)
+
     # Tunnel Id
     if rule.wc.tun_id_mask != 0:
         if rule.wc.tun_id_mask == UINT64_MAX:
@@ -1089,7 +1111,7 @@ def serialize_nxm_match(rule, buf, offset):
 
     # Pad
     pad_len = round_up(offset) - offset
-    ofproto_parser.msg_pack_into("%dx" % pad_len, buf, offset)
+    msg_pack_into("%dx" % pad_len, buf, offset)
 
     # The returned length, the match_len, does not include the pad
     return offset - old_offset
@@ -1103,7 +1125,7 @@ def nxm_put(buf, offset, header, rule):
 
 
 def round_up(length):
-    return (length + 7) / 8 * 8  # Round up to a multiple of 8
+    return (length + 7) // 8 * 8  # Round up to a multiple of 8
 
 
 class NXMatch(object):
@@ -1143,6 +1165,6 @@ class NXMatch(object):
                  self.hasmask(), self.length()))
 
     def put_header(self, buf, offset):
-        ofproto_parser.msg_pack_into(ofproto_v1_0.NXM_HEADER_PACK_STRING,
-                                     buf, offset, self.header)
+        msg_pack_into(ofproto_v1_0.NXM_HEADER_PACK_STRING,
+                      buf, offset, self.header)
         return struct.calcsize(ofproto_v1_0.NXM_HEADER_PACK_STRING)
