@@ -14,6 +14,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import six
+
 import base64
 import collections
 import logging
@@ -29,11 +31,16 @@ from . import ofproto_common
 
 LOG = logging.getLogger('ryu.ofproto.ofproto_parser')
 
+# This is merely for API compatibility on python2
+if six.PY3:
+    buffer = bytes
+
 
 def header(buf):
     assert len(buf) >= ofproto_common.OFP_HEADER_SIZE
     # LOG.debug('len %d bufsize %d', len(buf), ofproto.OFP_HEADER_SIZE)
-    return struct.unpack_from(ofproto_common.OFP_HEADER_PACK_STR, buffer(buf))
+    return struct.unpack_from(ofproto_common.OFP_HEADER_PACK_STR,
+                              six.binary_type(buf))
 
 
 _MSG_PARSERS = {}
@@ -104,14 +111,14 @@ def ofp_msg_from_jsondict(dp, jsondict):
     """
     parser = dp.ofproto_parser
     assert len(jsondict) == 1
-    for k, v in jsondict.iteritems():
+    for k, v in jsondict.items():
         cls = getattr(parser, k)
         assert issubclass(cls, MsgBase)
         return cls.from_jsondict(v, datapath=dp)
 
 
 class StringifyMixin(stringify.StringifyMixin):
-    _class_prefixes = ["OFP", "ONF", "MT"]
+    _class_prefixes = ["OFP", "ONF", "MT", "NX"]
 
     @classmethod
     def cls_from_jsondict_key(cls, k):
@@ -163,9 +170,11 @@ class MsgBase(StringifyMixin):
         self.buf = buffer(buf)
 
     def __str__(self):
-        buf = 'version: 0x%x msg_type 0x%x xid 0x%x ' % (self.version,
-                                                         self.msg_type,
-                                                         self.xid)
+        def hexify(x):
+            return str(None) if x is None else '0x%x' % x
+        buf = 'version: %s msg_type %s xid %s ' % (hexify(self.version),
+                                                   hexify(self.msg_type),
+                                                   hexify(self.xid))
         return buf + StringifyMixin.__str__(self)
 
     @classmethod
@@ -212,21 +221,6 @@ class MsgInMsgBase(MsgBase):
                                                   **additional_args)
 
 
-def msg_pack_into(fmt, buf, offset, *args):
-    if len(buf) < offset:
-        buf += bytearray(offset - len(buf))
-
-    if len(buf) == offset:
-        buf += struct.pack(fmt, *args)
-        return
-
-    needed_len = offset + struct.calcsize(fmt)
-    if len(buf) < needed_len:
-        buf += bytearray(needed_len - len(buf))
-
-    struct.pack_into(fmt, buf, offset, *args)
-
-
 def namedtuple(typename, fields, **kwargs):
     class _namedtuple(StringifyMixin,
                       collections.namedtuple(typename, fields, **kwargs)):
