@@ -18,7 +18,6 @@
 
 import netaddr
 from ryu.lib import hub
-from ryu.services.protocols.bgp.operator import ssh
 
 from ryu.services.protocols.bgp.core_manager import CORE_MANAGER
 from ryu.services.protocols.bgp.signals.emit import BgpSignalBus
@@ -161,6 +160,8 @@ class BGPSpeaker(object):
         self._peer_down_handler = peer_down_handler
         self._peer_up_handler = peer_up_handler
         if ssh_console:
+            from ryu.services.protocols.bgp.operator import ssh
+
             hub.spawn(ssh.SSH_CLI_CONTROLLER.start)
 
     def _notify_peer_down(self, peer):
@@ -408,7 +409,7 @@ class BGPSpeaker(object):
                 networks[NEXT_HOP] = \
                     str(netaddr.IPAddress(next_hop).ipv6())
 
-        call(func_name, **networks)
+        return call(func_name, **networks)
 
     def prefix_del(self, prefix, route_dist=None):
         """ This method deletes a advertized prefix.
@@ -491,6 +492,29 @@ class BGPSpeaker(object):
         show['format'] = format
         return call('operator.show', **show)
 
+    def neighbor_get(self, routetype, address, format='json'):
+        """ This method returns the BGP adj-RIB-in information in a json
+        format.
+
+        ``routetype`` This parameter is necessary for only received-routes
+        and sent-routes.
+
+          received-routes : paths received and not withdrawn by given peer
+
+          sent-routes : paths sent and not withdrawn to given peer
+
+        ``address`` specifies the IP address of the peer. It must be
+        the string representation of an IP address.
+
+        """
+        show = {}
+        if routetype == 'sent-routes' or routetype == 'received-routes':
+            show['params'] = ['neighbor', routetype, address, 'all']
+        else:
+            show['params'] = ['neighbor', 'received-routes', address, 'all']
+        show['format'] = format
+        return call('operator.show', **show)
+
     def _set_filter(self, filter_type, address, filters):
         assert filter_type in ('in', 'out'),\
             'filter type must be \'in\' or \'out\''
