@@ -90,8 +90,8 @@ def nofitication_factory(code, subcode):
 class BgpProtocol(Protocol, Activity):
     """Protocol that handles BGP messages.
     """
-    MESSAGE_MARKER = ('\xff\xff\xff\xff\xff\xff\xff\xff'
-                      '\xff\xff\xff\xff\xff\xff\xff\xff')
+    MESSAGE_MARKER = (b'\xff\xff\xff\xff\xff\xff\xff\xff'
+                      b'\xff\xff\xff\xff\xff\xff\xff\xff')
 
     def __init__(self, socket, signal_bus, is_reactive_conn=False):
         # Validate input.
@@ -320,7 +320,7 @@ class BgpProtocol(Protocol, Activity):
             # We should get default marker since we are not supporting any
             # authentication.
             if (auth != BgpProtocol.MESSAGE_MARKER):
-                LOG.error('Invalid message marker received: %s' % auth)
+                LOG.error('Invalid message marker received: %s', auth)
                 raise bgp.NotSync()
 
             # Check if we have valid bgp message length.
@@ -372,8 +372,8 @@ class BgpProtocol(Protocol, Activity):
         self._send_with_lock(notification)
         self._signal_bus.bgp_error(self._peer, code, subcode, reason)
         if len(self._localname):
-            LOG.error('Sent notification to %r >> %s' % (self._localname,
-                                                         notification))
+            LOG.error('Sent notification to %r >> %s', self._localname,
+                      notification)
         self._socket.close()
 
     def _send_with_lock(self, msg):
@@ -393,12 +393,11 @@ class BgpProtocol(Protocol, Activity):
         self._send_with_lock(msg)
 
         if msg.type == BGP_MSG_NOTIFICATION:
-            LOG.error('Sent notification to %s >> %s' %
-                      (self._remotename, msg))
+            LOG.error('Sent notification to %s >> %s', self._remotename, msg)
 
             self._signal_bus.bgp_notification_sent(self._peer, msg)
         else:
-            LOG.debug('Sent msg to %s >> %s' % (self._remotename, msg))
+            LOG.debug('Sent msg to %s >> %s', self._remotename, msg)
 
     def stop(self):
         Activity.stop(self)
@@ -430,7 +429,7 @@ class BgpProtocol(Protocol, Activity):
         message except for *Open* and *Notification* message. On receiving
         *Notification* message we close connection with peer.
         """
-        LOG.debug('Received msg from %s << %s' % (self._remotename, msg))
+        LOG.debug('Received msg from %s << %s', self._remotename, msg)
 
         # If we receive open message we try to bind to protocol
         if (msg.type == BGP_MSG_OPEN):
@@ -478,7 +477,7 @@ class BgpProtocol(Protocol, Activity):
                 self._signal_bus.bgp_notification_received(self._peer, msg)
             # If we receive notification message
             LOG.error('Received notification message, hence closing '
-                      'connection %s' % msg)
+                      'connection %s', msg)
             self._socket.close()
             return
 
@@ -504,23 +503,23 @@ class BgpProtocol(Protocol, Activity):
         neg_timer = min(self._holdtime, peer_holdtime)
         if neg_timer < self._holdtime:
             LOG.info('Negotiated hold time (%s) is lower then '
-                     'configured/default (%s).' % (neg_timer, self._holdtime))
+                     'configured/default (%s).', neg_timer, self._holdtime)
         # We use negotiated timer value.
         self._holdtime = neg_timer
         self._keepalive = self._create_timer('Keepalive Timer',
                                              self._send_keepalive)
-        interval = self._holdtime / 3
+        interval = self._holdtime // 3
         self._keepalive.start(interval, now=False)
         # Setup the expire timer.
         self._expiry = self._create_timer('Holdtime Timer', self._expired)
         self._expiry.start(self._holdtime, now=False)
         LOG.debug('Started keep-alive and expire timer for negotiated hold'
-                  'time %s' % self._holdtime)
+                  'time %s', self._holdtime)
 
     def _expired(self):
         """Hold timer expired event handler.
         """
-        LOG.info('Negotiated hold time %s expired.' % self._holdtime)
+        LOG.info('Negotiated hold time %s expired.', self._holdtime)
         code = BGP_ERROR_HOLD_TIMER_EXPIRED
         subcode = BGP_ERROR_SUB_HOLD_TIMER_EXPIRED
         self.send_notification(code, subcode)
