@@ -15,6 +15,8 @@
 
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
+import six
+
 import binascii
 import unittest
 from nose.tools import *
@@ -27,6 +29,9 @@ from ryu.ofproto import ofproto_v1_0, ofproto_v1_0_parser
 import logging
 LOG = logging.getLogger(__name__)
 
+if six.PY3:
+    buffer = bytes
+
 
 class TestOfproto_Parser(unittest.TestCase):
     def setUp(self):
@@ -164,7 +169,7 @@ class TestMsgBase(unittest.TestCase):
         version = ofproto_v1_0.OFP_VERSION
         msg_len = ofproto_v1_0.OFP_HEADER_SIZE
         xid = 2183948390
-        data = '\x00\x01\x02\x03'
+        data = b'\x00\x01\x02\x03'
 
         fmt = ofproto_v1_0.OFP_HEADER_PACK_STR
         buf = struct.pack(fmt, version, msg_type, msg_len, xid) \
@@ -223,43 +228,6 @@ class TestMsgBase(unittest.TestCase):
         ok_(self._test_serialize())
 
 
-class TestMsgPackInto(unittest.TestCase):
-    """ Test case for ofproto_parser.msg_pack_into
-    """
-
-    def _test_msg_pack_into(self, offset_type='e'):
-        fmt = '!HH'
-        len_ = struct.calcsize(fmt)
-        buf = bytearray(len_)
-        offset = len_
-        arg1 = 1
-        arg2 = 2
-
-        if offset_type == 'l':
-            offset += 1
-        elif offset_type == 'g':
-            offset -= 1
-
-        ofproto_parser.msg_pack_into(fmt, buf, offset, arg1, arg2)
-
-        check_offset = len(buf) - len_
-        res = struct.unpack_from(fmt, buffer(buf), check_offset)
-
-        eq_(arg1, res[0])
-        eq_(arg2, res[1])
-
-        return True
-
-    def test_msg_pack_into(self):
-        ok_(self._test_msg_pack_into())
-
-    def test_msg_pack_into_less(self):
-        ok_(self._test_msg_pack_into('l'))
-
-    def test_msg_pack_into_greater(self):
-        ok_(self._test_msg_pack_into('g'))
-
-
 class TestMsgStrAttr(unittest.TestCase):
     """ Test case for ofproto_parser.msg_str_attr
     """
