@@ -17,10 +17,10 @@
 
 import unittest
 import logging
+import six
 import struct
 from struct import *
 from nose.tools import *
-from nose.plugins.skip import Skip, SkipTest
 from ryu.ofproto import ether, inet
 from ryu.lib.packet.packet import Packet
 from ryu.lib.packet.tcp import tcp
@@ -44,7 +44,7 @@ class Test_tcp(unittest.TestCase):
     window_size = 2048
     csum = 12345
     urgent = 128
-    option = '\x01\x02\x03\x04'
+    option = b'\x01\x02\x03\x04'
 
     t = tcp(src_port, dst_port, seq, ack, offset, bits,
             window_size, csum, urgent, option)
@@ -98,7 +98,7 @@ class Test_tcp(unittest.TestCase):
         t = tcp(self.src_port, self.dst_port, self.seq, self.ack,
                 offset, self.bits, self.window_size, csum, self.urgent)
         buf = t.serialize(bytearray(), prev)
-        res = struct.unpack(tcp._PACK_STR, str(buf))
+        res = struct.unpack(tcp._PACK_STR, six.binary_type(buf))
 
         eq_(res[0], self.src_port)
         eq_(res[1], self.dst_port)
@@ -120,7 +120,7 @@ class Test_tcp(unittest.TestCase):
     def test_serialize_option(self):
         offset = 6
         csum = 0
-        option = '\x01\x02'
+        option = b'\x01\x02'
 
         src_ip = '192.168.10.1'
         dst_ip = '192.168.100.1'
@@ -155,7 +155,7 @@ class Test_tcp(unittest.TestCase):
         eq_(res[8], 0)
 
         # with option, without offset
-        t = tcp(option='\x01\x02\x03')
+        t = tcp(option=b'\x01\x02\x03')
         buf = t.serialize(bytearray(), prev)
         res = struct.unpack(tcp._PACK_STR + '4s', buf)
 
@@ -167,10 +167,10 @@ class Test_tcp(unittest.TestCase):
         eq_(res[5], 0)
         eq_(res[6], 0)
         eq_(res[8], 0)
-        eq_(res[9], '\x01\x02\x03\x00')
+        eq_(res[9], b'\x01\x02\x03\x00')
 
         # with option, with long offset
-        t = tcp(offset=7, option='\x01\x02\x03')
+        t = tcp(offset=7, option=b'\x01\x02\x03')
         buf = t.serialize(bytearray(), prev)
         res = struct.unpack(tcp._PACK_STR + '8s', buf)
 
@@ -182,7 +182,7 @@ class Test_tcp(unittest.TestCase):
         eq_(res[5], 0)
         eq_(res[6], 0)
         eq_(res[8], 0)
-        eq_(res[9], '\x01\x02\x03\x00\x00\x00\x00\x00')
+        eq_(res[9], b'\x01\x02\x03\x00\x00\x00\x00\x00')
 
     def test_json(self):
         jsondict = self.t.to_jsondict()
