@@ -18,6 +18,7 @@
 
 import unittest
 import logging
+import six
 import struct
 import inspect
 
@@ -109,7 +110,7 @@ class Test_vrrpv2(unittest.TestCase):
         buf = vrrp_.serialize(bytearray(), prev)
         pack_str = vrrp.vrrpv2._PACK_STR + '4sII'
         pack_len = struct.calcsize(pack_str)
-        res = struct.unpack(pack_str, str(buf))
+        res = struct.unpack(pack_str, six.binary_type(buf))
         eq_(res[0], vrrp.vrrp_to_version_type(vrrp.VRRP_VERSION_V2, type_))
         eq_(res[1], vrid)
         eq_(res[2], priority)
@@ -135,7 +136,7 @@ class Test_vrrpv2(unittest.TestCase):
         primary_ip = '192.168.0.2'
         p0 = self.vrrpv2.create_packet(primary_ip)
         p0.serialize()
-        p1 = packet.Packet(str(p0.data))
+        p1 = packet.Packet(six.binary_type(p0.data))
         p1.serialize()
         eq_(p0.data, p1.data)
 
@@ -272,7 +273,7 @@ class Test_vrrpv3_ipv4(unittest.TestCase):
         print(len(buf), type(buf), buf)
         pack_str = vrrp.vrrpv3._PACK_STR + '4s'
         pack_len = struct.calcsize(pack_str)
-        res = struct.unpack(pack_str, str(buf))
+        res = struct.unpack(pack_str, six.binary_type(buf))
         eq_(res[0], vrrp.vrrp_to_version_type(vrrp.VRRP_VERSION_V3, type_))
         eq_(res[1], vrid)
         eq_(res[2], priority)
@@ -300,7 +301,7 @@ class Test_vrrpv3_ipv4(unittest.TestCase):
         primary_ip = '192.168.0.2'
         p0 = self.vrrpv3.create_packet(primary_ip)
         p0.serialize()
-        p1 = packet.Packet(str(p0.data))
+        p1 = packet.Packet(six.binary_type(p0.data))
         p1.serialize()
         eq_(p0.data, p1.data)
 
@@ -437,7 +438,7 @@ class Test_vrrpv3_ipv6(unittest.TestCase):
         print(len(buf), type(buf), buf)
         pack_str = vrrp.vrrpv3._PACK_STR + '16s'
         pack_len = struct.calcsize(pack_str)
-        res = struct.unpack(pack_str, str(buf))
+        res = struct.unpack(pack_str, six.binary_type(buf))
         eq_(res[0], vrrp.vrrp_to_version_type(vrrp.VRRP_VERSION_V3, type_))
         eq_(res[1], vrid)
         eq_(res[2], priority)
@@ -466,7 +467,7 @@ class Test_vrrpv3_ipv6(unittest.TestCase):
         p0 = self.vrrpv3.create_packet(primary_ip)
         p0.serialize()
         print(len(p0.data), p0.data)
-        p1 = packet.Packet(str(p0.data))
+        p1 = packet.Packet(six.binary_type(p0.data))
         p1.serialize()
         print(len(p0.data), p0.data)
         print(len(p1.data), p1.data)
